"""Test user-defined (custom) operator
"""

from hysop.operator.custom import Custom
from hysop import Field, Discretization, Box, Simulation, IOParams, IO
import numpy as np
import os

d2d = Discretization([33, 65], [2, 0])
d3d = Discretization([33, 65, 33], [2, 0, 1])


def func3d(sim, f_in, f_out, d=None):
    """A test function, for a 3d domain,
    no diagnostics
    """
    nbc = f_in[0].nb_components
    for d in xrange(nbc):
        f_out[0][d] = d * f_in[0][d] + + f_in[1][0] + np.cos(sim.time)


def func3d_with_diag(sim, f_in, f_out, diagnostics):
    """A test function, for a 3d domain,
    no diagnostics
    """
    nbc = f_in[0].nb_components
    for d in xrange(nbc):
        f_out[0][d] = d * f_in[0][d] + f_in[1][0] + np.cos(sim.time)
    diagnostics[0, 0] = sim.time
    diagnostics[0, 1] = f_out[0].data[0].min()
    diagnostics[0, 1] = f_out[0].data[0].max()


def init_custom(dim, discr, func, do_write=False):
    """Test build and apply, without
    writer.
    """
    dom = Box(length=[1., ] * dim)
    v1 = Field(name='v1', is_vector=True, domain=dom)
    v2 = Field(name='v2', is_vector=False, domain=dom)
    v3 = Field(name='v3', is_vector=True, domain=dom)
    topo = dom.create_topology(discr)
    v1.randomize(topo)
    v2.randomize(topo)
    if do_write:
        iop = IOParams('/tmp/hysop_custom_test.dat', fileformat=IO.ASCII)
        d_shape = (1, 3)
    else:
        iop = None
        d_shape = None
    op = Custom(in_fields=[v1, v2], out_fields=[v3],
                variables=[v1, v2, v3], discretization=discr,
                function=func, io_params=iop,
                diagnostics_shape=d_shape)
    op.discretize()
    op.setup()
    sim = Simulation(nb_iter=10)
    sim.initialize()
    nbc = v1.nb_components
    vd1 = v1.discretize(topo)
    vd2 = v2.discretize(topo)
    vd3 = v3.discretize(topo)
    while not sim.is_over:
        op.apply(sim)
        for d in xrange(nbc):
            tmp1 = d * vd1[d] + + vd2[0] + np.cos(sim.time)
            assert np.allclose(vd3[d], tmp1)
        sim.advance()
    if do_write:
        assert os.path.isfile(iop.filename)


def test_custom_op_1():
    """Test build and apply, without
    writer.
    """
    init_custom(3, d3d, func3d)


def test_custom_op_2():
    """Test build and apply, with
    output in a file
    """
    init_custom(3, d3d, func3d_with_diag, True)
