"""Tests for diffusion and curl_diffusion operators.
"""
# -*- coding: utf-8 -*-

import hysop as pp
from hysop.operator.diffusion import Diffusion, CurlAndDiffusion
from hysop.problem.simulation import Simulation
from hysop.tools.parameters import Discretization
import numpy as np
from hysop.tools.numpywrappers import npw
import math
from hysop import testsenv
from hysop.fields.tests.func_for_tests import v_TG, w_TG
pi = math.pi
sin = np.sin
cos = np.cos
# Physical Domain description
dim = 3
LL = 2 * pi * npw.ones((dim))
cc = 2 * pi / LL
d3D = Discretization([33, 33, 33])
d2D = Discretization([33, 33])


def computeVort2D(res, x, y, t):
    # todo ...
    res[0][...] = 4 * pi ** 2 * (cos(x * cc[0]) * sin(y * cc[1])) * \
        (1. / LL[0] ** 2 + 1. / LL[1] ** 2)
    return res


@testsenv.fftw_failed
def test_diffusion_3d():
    """Vector field diffusion, 3d domain"""
    dom = pp.Box(length=LL)
    vorticity = pp.Field(domain=dom, formula=w_TG,
                         name='Vorticity', is_vector=True)
    diff = Diffusion(viscosity=0.3, vorticity=vorticity, discretization=d3D)
    diff.discretize()
    diff.setup()
    topo = diff.discrete_fields[vorticity].topology
    simu = Simulation(end=0.1, time_step=0.01)
    vorticity.initialize(topo=topo)
    diff.apply(simu)
    diff.finalize()


@testsenv.fftw_failed
def test_diffusion_2d():
    """Vector field diffusion 2d domain"""
    dom = pp.Box(length=LL[:2])
    vorticity = pp.Field(domain=dom, formula=computeVort2D, name='Vorticity')
    diff = Diffusion(viscosity=0.3, vorticity=vorticity, discretization=d2D)
    diff.discretize()
    diff.setup()
    topo = diff.discrete_fields[vorticity].topology
    simu = Simulation(end=0.1, time_step=0.01)
    vorticity.initialize(topo=topo)
    diff.apply(simu)
    diff.finalize()


@testsenv.fftw_failed
def test_curl_and_diffusion_3d():
    """Vector field diffusion, 3d domain"""
    dom = pp.Box(length=LL)
    vorticity_ref = pp.Field(domain=dom, name='Wref', formula=w_TG,
                             is_vector=True)
    vorticity = pp.Field(domain=dom, name='Vorticity', is_vector=True)
    velocity = pp.Field(domain=dom, name='Velocity', formula=v_TG,
                        is_vector=True)

    diff_0 = Diffusion(viscosity=0.3, vorticity=vorticity_ref,
                       discretization=d3D)
    diff_0.discretize()
    diff_0.setup()
    topo = diff_0.discrete_fields[vorticity_ref].topology
    diff = CurlAndDiffusion(viscosity=0.3, velocity=velocity,
                            vorticity=vorticity, discretization=d3D)
    diff.discretize()
    diff.setup()

    simu = Simulation(end=0.1, time_step=0.01)
    vorticity_ref.initialize(topo=topo)
    velocity.initialize(topo=topo)
    simu.initialize()
    wd = vorticity.discretize(topo).data
    wd_ref = vorticity_ref.discretize(topo).data
    diff_0.apply(simu)
    diff.apply(simu)
    for i in xrange(3):
        assert np.allclose(wd[i], wd_ref[i])

    diff_0.finalize()
